"""
Example of Duo Accounts API get child account edition
"""

import duo_client
import getpass

DUO_EDITIONS = {
        "ENTERPRISE": "Duo Essentials",
        "PLATFORM": "Duo Advantage",
        "BEYOND": "Duo Premier",
        "PERSONAL": "Duo Free"
}

def _get_user_input(prompt, secure=False):
    """Read information from STDIN, using getpass when sensitive information should not be echoed to tty"""
    if secure is True:
        return getpass.getpass(prompt)
    else:
        return input(prompt)


def prompt_for_credentials() -> dict:
    """Collect required API credentials from command line prompts"""

    ikey = _get_user_input('Duo Accounts API integration key ("DI..."): ')
    skey = _get_user_input('Duo Accounts API integration secret key: ', secure=True)
    host = _get_user_input('Duo Accounts API hostname ("api-....duosecurity.com"): ')
    account_id = _get_user_input('Child account ID: ')

    return {
            "ikey": ikey,
            "skey": skey,
            "host": host,
            "account_id": account_id,
    }


def main():
    """Main program entry point"""

    inputs = prompt_for_credentials()

    account_admin_api = duo_client.admin.AccountAdmin(**inputs)

    print(f"Getting edition for account ID {inputs['account_id']}...")
    result = account_admin_api.get_edition()
    if 'edition' not in result:
        print(f"An error occurred while getting edition for account {inputs['account_id']}")
        print(f"Error message: {result}")
    else:
        print(f"The current Duo Edition for account {inputs['account_id']} is '{result['edition']}' " +
              f"[{DUO_EDITIONS[result['edition']]}]")


if __name__ == '__main__':
    main()



#!/usr/bin/python
import os
import sys
import time

import duo_client
from configparser import ConfigParser
from urllib.parse import urlparse


class BaseLog:

    def __init__(self, admin_api, path, logname):
        self.admin_api = admin_api
        self.path = path
        self.logname = logname

        self.mintime = 0
        self.events = []

    def get_events(self):
        raise NotImplementedError

    def print_events(self):
        raise NotImplementedError

    def get_last_timestamp_path(self):
        """
        Returns the path to the file containing the timestamp of the last
        event fetched.
        """
        filename = self.logname + "_last_timestamp_" + self.admin_api.host
        path = os.path.join(self.path, filename)
        return path

    def get_mintime(self):
        """
            Updates self.mintime which is the minimum timestamp of
            log events we want to fetch.
            self.mintime is > all event timestamps we have already fetched.
        """
        try:
            # Only fetch events that come after timestamp of last event
            path = self.get_last_timestamp_path()
            self.mintime = int(open(path).read().strip()) + 1
        except IOError:
            pass

    def write_last_timestamp(self):
        """
            Store last_timestamp so that we don't fetch the same events again
        """
        if not self.events:
            # Do not update last_timestamp
            return

        last_timestamp = 0
        for event in self.events:
            last_timestamp = max(last_timestamp, event['timestamp'])

        path = self.get_last_timestamp_path()
        f = open(path, "w")
        f.write(str(last_timestamp))
        f.close()

    def run(self):
        """
        Fetch new log events and print them.
        """
        self.events = []
        self.get_mintime()
        self.get_events()
        self.print_events()
        self.write_last_timestamp()


class AdministratorLog(BaseLog):
    def __init__(self, admin_api, path):
        BaseLog.__init__(self, admin_api, path, "administrator")

    def get_events(self):
        self.events = self.admin_api.get_administrator_log(
            mintime=self.mintime,
        )

    def print_events(self):
        """
        Print events in a format suitable for Splunk.
        """
        for event in self.events:
            event['ctime'] = time.ctime(event['timestamp'])
            event['actionlabel'] = {
                'admin_login': "Admin Login",
                'admin_create': "Create Admin",
                'admin_update': "Update Admin",
                'admin_delete': "Delete Admin",
                'customer_update': "Update Customer",
                'group_create': "Create Group",
                'group_update': "Update Group",
                'group_delete': "Delete Group",
                'integration_create': "Create Integration",
                'integration_update': "Update Integration",
                'integration_delete': "Delete Integration",
                'phone_create': "Create Phone",
                'phone_update': "Update Phone",
                'phone_delete': "Delete Phone",
                'user_create': "Create User",
                'user_update': "Update User",
                'user_delete': "Delete User"}.get(
                event['action'], event['action'])

            fmtstr = '%(timestamp)s,' \
                     'host="%(host)s", ' \
                     'eventtype="%(eventtype)s", ' \
                     'username="%(username)s", ' \
                     'action="%(actionlabel)s"'
            if event['object']:
                fmtstr += ', object="%(object)s"'
            if event['description']:
                fmtstr += ', description="%(description)s"'

            print(fmtstr % event)


class AuthenticationLog(BaseLog):
    def __init__(self, admin_api, path):
        BaseLog.__init__(self, admin_api, path, "authentication")

    def get_events(self):
        self.events = self.admin_api.get_authentication_log(
            mintime=self.mintime,
        )

    def print_events(self):
        """
        Print events in a format suitable for Splunk.
        """
        for event in self.events:
            event['ctime'] = time.ctime(event['timestamp'])

            fmtstr = (
                '%(timestamp)s,'
                'host="%(host)s", '
                'eventtype="%(eventtype)s", '
                'username="%(username)s", '
                'factor="%(factor)s", '
                'result="%(result)s", '
                'reason="%(reason)s", '
                'ip="%(ip)s", '
                'integration="%(integration)s", '
                'newenrollment="%(new_enrollment)s"'
            )

            print(fmtstr % event)


class TelephonyLog(BaseLog):
    def __init__(self, admin_api, path):
        BaseLog.__init__(self, admin_api, path, "telephony")

    def get_events(self):
        self.events = self.admin_api.get_telephony_log(
            mintime=self.mintime,
        )

    def print_events(self):
        """
        Print events in a format suitable for Splunk.
        """
        for event in self.events:
            event['ctime'] = time.ctime(event['timestamp'])
            event['host'] = self.admin_api.host

            fmtstr = '%(timestamp)s,' \
                     'host="%(host)s", ' \
                     'eventtype="%(eventtype)s", ' \
                     'context="%(context)s", ' \
                     'type="%(type)s", ' \
                     'phone="%(phone)s", ' \
                     'credits="%(credits)s"'

            print(fmtstr % event)


def admin_api_from_config(config_path):
    """
    Return a duo_client.Admin object created using the parameters
    stored in a config file.
    """
    config = ConfigParser()
    config.read(config_path)
    config_d = dict(config.items('duo'))
    ca_certs = config_d.get("ca_certs", None)
    if ca_certs is None:
        ca_certs = config_d.get("ca", None)

    ret = duo_client.Admin(
        ikey=config_d['ikey'],
        skey=config_d['skey'],
        host=config_d['host'],
        ca_certs=ca_certs,
    )

    http_proxy = config_d.get("http_proxy", None)
    if http_proxy is not None:
        proxy_parsed = urlparse(http_proxy)
        proxy_host = proxy_parsed.hostname
        proxy_port = proxy_parsed.port
        ret.set_proxy(host = proxy_host, port = proxy_port)

    return ret


def main():
    if len(sys.argv) == 1:
        config_path = os.path.abspath(__file__)
        config_path = os.path.dirname(config_path)
        config_path = os.path.join(config_path, "duo.conf")
    else:
        config_path = os.path.abspath(sys.argv[1])

    admin_api = admin_api_from_config(config_path)

    # Use the directory of the config file to store the last event tstamps
    path = os.path.dirname(config_path)

    for logclass in (AdministratorLog, AuthenticationLog, TelephonyLog):
        log = logclass(admin_api, path)
        log.run()


if __name__ == '__main__':
    main()



from duo_client.https_wrapper import CertValidatingHTTPSConnection
import unittest
from unittest import mock
import ssl

class TestSSLContextCreation(unittest.TestCase):
    """ Test that the SSL context used to wrap sockets is configured correctly """
    def test_no_ca_certs(self):
        conn = CertValidatingHTTPSConnection('api-fakehost.duosecurity.com')
        self.assertEqual(conn.default_ssl_context.verify_mode, ssl.CERT_NONE)  # noqa: DUO122, testing insecure context

    @mock.patch('ssl.SSLContext.load_verify_locations')
    def test_with_ca_certs(self, mock_load):
        mock_load.return_value = None
        conn = CertValidatingHTTPSConnection('api-fakehost.duosecurity.com', ca_certs='cafilepath')
        self.assertEqual(conn.default_ssl_context.verify_mode, ssl.CERT_REQUIRED)
        mock_load.assert_called_with(cafile='cafilepath')

    @mock.patch('ssl.SSLContext.load_cert_chain')
    def test_with_certfile(self, mock_load):
        mock_load.return_value = None
        CertValidatingHTTPSConnection('api-fakehost.duosecurity.com', cert_file='certfilepath')
        mock_load.assert_called_with('certfilepath', None)

    def test_ssl2_ssl3_off(self):
        conn = CertValidatingHTTPSConnection('api-fakehost.duosecurity.com')
        self.assertEqual(conn.default_ssl_context.options & ssl.OP_NO_SSLv2, ssl.OP_NO_SSLv2)
        self.assertEqual(conn.default_ssl_context.options & ssl.OP_NO_SSLv3, ssl.OP_NO_SSLv3)

    @mock.patch('socket.socket.connect')
    def test_server_hostname(self, mock_connect):
        hostname = 'api-fakehost.duosecurity.com'
        conn = CertValidatingHTTPSConnection(hostname)
        conn.connect()
        self.assertEqual(conn.sock.server_hostname, hostname)

    @mock.patch('socket.socket.connect')
    def test_server_hostname_with_port(self, mock_connect):
        hostname = 'api-fakehost.duosecurity.com'
        conn = CertValidatingHTTPSConnection(f'{hostname}:443')
        conn.connect()
        self.assertEqual(conn.sock.server_hostname, hostname)



import unittest
from .. import util
import duo_client.admin
import os


class TestAdmin(unittest.TestCase):

    TEST_RESOURCES_DIR = dir_path = os.path.join(
        os.path.abspath(os.curdir), 'tests', 'resources')

    def setUp(self):
        self.client = duo_client.admin.Admin(
            'test_ikey', 'test_akey', 'example.com')
        self.client.account_id = 'DA012345678901234567'
        # monkeypatch client's _connect()
        self.client._connect = lambda: util.MockHTTPConnection()

        # if you are wanting to simulate getting lists of objects
        # rather than a single object
        self.client_list = duo_client.admin.Admin(
            'test_ikey', 'test_akey', 'example.com')
        self.client_list.account_id = 'DA012345678901234567'
        self.client_list._connect = \
            lambda: util.MockHTTPConnection(data_response_should_be_list=True)

        # if you are wanting to get a response from a call to get
        # authentication logs
        self.client_authlog = duo_client.admin.Admin(
            'test_ikey', 'test_akey', 'example.com')
        self.client_authlog.account_id = 'DA012345678901234567'
        self.client_authlog._connect = \
            lambda: util.MockHTTPConnection(data_response_from_get_authlog=True)

        # client to simulate basic structure of a call to fetch Duo Trust
        # Monitor events.
        self.client_dtm = duo_client.admin.Admin(
            'test_ikey',
            'test_akey',
            'example.com',
        )
        self.client_dtm.account_id = 'DA012345678901234567'
        self.client_dtm._connect = \
            lambda: util.MockHTTPConnection(data_response_from_get_dtm_events=True)

        self.items_response_client = duo_client.admin.Admin(
            'test_ikey', 'test_akey', 'example.com')
        self.items_response_client.account_id = 'DA012345678901234567'
        self.items_response_client._connect = \
            lambda: util.MockHTTPConnection(data_response_from_get_items=True)


if __name__ == '__main__':
    unittest.main()



"""
Example of creating an integration in a child account using parent account credentials

The key to successfully interacting with child accounts via the parent account APIs is
pairing the parent account API IKEY/SKEY combination with the api-host of the child account.
Once that connection is established, the child account ID must be passed along with all API interactions.
The duo_client SDK makes that easy by allowing the setting of the child account ID as an instance variable.
"""

import sys
import getpass
import duo_client

# Create an interator to be used by the interactive terminal prompt
argv_iter = iter(sys.argv[1:])


def _get_next_arg(prompt, secure=False):
    """Read information from STDIN, using getpass when sensitive information should not be echoed to tty"""
    try:
        return next(argv_iter)
    except StopIteration:
        if secure is True:
            return getpass.getpass(prompt)
        else:
            return input(prompt)


def prompt_for_credentials() -> dict:
    """Collect required API credentials from command line prompts

    :return: dictionary containing Duo Accounts API ikey, skey and hostname strings
    """
    answers = {'ikey':       _get_next_arg('Duo Accounts API integration key ("DI..."): '),
               'skey':       _get_next_arg('Duo Accounts API integration secret key: ', secure=True),
               'host':       _get_next_arg('Duo API hostname of child account ("api-....duosecurity.com"): '),
               'account_id': _get_next_arg('Child account ID: '), 
               'app_name': _get_next_arg('New application name: '),
               'app_type':   _get_next_arg('New application type: ')}
    return answers


def create_child_integration(inputs: dict):
    """Create new application integration in child account via the parent account API"""

    # First create a duo_client.Admin instance using the parent account ikey/sky along with the child account api-host
    account_client = duo_client.Admin(ikey=inputs['ikey'], skey=inputs['skey'], host=inputs['host'])
    # Next assign the child account ID to the duo_client.Admin instance variable.
    account_client.account_id = inputs['account_id']
    # Now all API calls made via this instance will contain all of the minimum requirements to interact with the
    # child account.

    # Here only the two required arguments (name and type) are passed.
    # Normally, much more information would be provided. The type of additional information
    # varies by the type of application integration.
    try:
        new_app = account_client.create_integration(
                name=inputs['app_name'],
                integration_type=inputs['app_type'],
        )
        print(f"New application {inputs['app_name']} (ID: {new_app['integration_key']}) was created successfully.")
    except RuntimeError as e_str:
        # Any failure of the API call results in a generic Runtime Error
        print(f"An error occurred while creating the new application: {e_str}")


def main():
    """Main program entry point"""
    inputs = prompt_for_credentials()
    create_child_integration(inputs)


if __name__ == '__main__':
    main()



import json
from .. import util
import duo_client.admin
from .base import TestAdmin


class TestSettings(TestAdmin):

    def test_update_settings(self):
        """ Test updating settings
        """
        self.maxDiff = None
        response = self.client_list.update_settings(
            lockout_threshold=10,
            lockout_expire_duration=60,
            inactive_user_expiration=30,
            pending_deletion_days=5,
            log_retention_days=180,
            sms_batch=5,
            sms_expiration=60,
            sms_refresh=True,
            sms_message='test_message',
            fraud_email='test@example.com',
            fraud_email_enabled=True,
            keypress_confirm='0',
            keypress_fraud='9',
            timezone='UTC',
            telephony_warning_min=50,
            caller_id='+15035551000',
            user_telephony_cost_max=10,
            minimum_password_length=12,
            password_requires_upper_alpha=True,
            password_requires_lower_alpha=True,
            password_requires_numeric=True,
            password_requires_special=True,
            helpdesk_bypass="allow",
            helpdesk_bypass_expiration=60,
            helpdesk_message="test_message",
            helpdesk_can_send_enroll_email=True,
            reactivation_url="https://www.example.com",
            reactivation_integration_key='DINTEGRATIONKEYTEST0',
            security_checkup_enabled=True,
            user_managers_can_put_users_in_bypass=False,
            email_activity_notification_enabled=True,
            push_activity_notification_enabled=True,
            unenrolled_user_lockout_threshold=100,
            enrollment_universal_prompt_enabled=True,
        )
        response = response[0]
        self.assertEqual(response['method'], 'POST')
        self.assertEqual(response['uri'], '/admin/v1/settings')
        self.assertEqual(
            json.loads(response['body']),
            {
                'account_id': self.client.account_id,
                'lockout_threshold': '10',
                'lockout_expire_duration': '60',
                'inactive_user_expiration': '30',
                'log_retention_days': '180',
                'sms_batch': '5',
                'sms_expiration': '60',
                'sms_refresh': '1',
                'sms_message': 'test_message',
                'fraud_email': 'test@example.com',
                'fraud_email_enabled': '1',
                'keypress_confirm': '0',
                'keypress_fraud': '9',
                'timezone': 'UTC',
                'telephony_warning_min': '50',
                'caller_id': '+15035551000',
                'user_telephony_cost_max': '10',
                'pending_deletion_days': '5',
                'minimum_password_length': '12',
                'password_requires_upper_alpha': '1',
                'password_requires_lower_alpha': '1',
                'password_requires_numeric': '1',
                'password_requires_special': '1',
                'helpdesk_bypass': 'allow',
                'helpdesk_bypass_expiration': '60',
                'helpdesk_message': 'test_message',
                'helpdesk_can_send_enroll_email': '1',
                'reactivation_url': 'https://www.example.com',
                'reactivation_integration_key': 'DINTEGRATIONKEYTEST0',
                'security_checkup_enabled': '1',
                'user_managers_can_put_users_in_bypass': '0',
                'email_activity_notification_enabled': '1',
                'push_activity_notification_enabled': '1',
                'unenrolled_user_lockout_threshold': '100',
                'enrollment_universal_prompt_enabled': '1',
            })



import json
import collections
import urllib.parse

from json import JSONEncoder
import duo_client

class MockObjectJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        return getattr(obj.__class__, "to_json")(obj)

# put params in a dict to avoid inconsistent ordering
def params_to_dict(param_str):
    param_dict = collections.defaultdict(list)
    for (key, val) in (param.split('=') for param in param_str.split('&')):
        param_dict[key].append(urllib.parse.unquote(val))
    return param_dict


class MockHTTPConnection(object):
    """
    Mock HTTP(S) connection that returns a dummy JSON response.
    """
    status = 200            # success!

    def __init__(
        self,
        data_response_should_be_list=False,
        data_response_from_get_authlog=False,
        data_response_from_get_dtm_events=False,
        data_response_from_get_items=False,
    ):
        # if a response object should be a list rather than
        # a dict, then set this flag to true
        self.data_response_should_be_list = data_response_should_be_list
        self.data_response_from_get_authlog = data_response_from_get_authlog
        self.data_response_from_get_dtm_events = data_response_from_get_dtm_events
        self.data_response_from_get_items = data_response_from_get_items

    def dummy(self):
        return self

    _connect = _disconnect = close = getresponse = dummy

    def read(self):
        response = self.__dict__

        if self.data_response_should_be_list:
            response = [self.__dict__]

        if self.data_response_from_get_authlog:
            response['authlogs'] = []

        if self.data_response_from_get_items:
            response['items'] = []

        if self.data_response_from_get_dtm_events:
            response['events'] = [{"foo": "bar"}, {"bar": "foo"}]

        return json.dumps({"stat":"OK", "response":response},
                              cls=MockObjectJsonEncoder)

    def request(self, method, uri, body, headers):
        self.method = method
        self.uri = uri
        self.body = body

        self.headers = {}
        for k, v in headers.items():
            if isinstance(k, bytes):
                k = k.decode('ascii')
            if isinstance(v, bytes):
                v = v.decode('ascii')
            self.headers[k] = v


class MockJsonObject(object):
    def to_json(self):
        return {'id': id(self)}

class CountingClient(duo_client.client.Client):
    def __init__(self, *args, **kwargs):
        super(CountingClient, self).__init__(*args, **kwargs)
        self.counter = 0

    def _make_request(self, *args, **kwargs):
        self.counter += 1
        return super(CountingClient, self)._make_request(*args, **kwargs)


class MockPagingHTTPConnection(MockHTTPConnection):
    def __init__(self, objects=None):
        if objects is not None:
            self.objects = objects

    def dummy(self):
        return self

    _connect = _disconnect = close = getresponse = dummy

    def read(self):
        metadata = {}
        metadata['total_objects'] = len(self.objects)
        if self.offset + self.limit < len(self.objects):
            metadata['next_offset'] = self.offset + self.limit
        if self.offset > 0:
            metadata['prev_offset'] = max(self.offset-self.limit, 0)

        return json.dumps(
                {"stat":"OK",
                "response": self.objects[self.offset: self.offset+self.limit],
                "metadata": metadata},
                cls=MockObjectJsonEncoder)

    def request(self, method, uri, body, headers):
        self.method = method
        self.uri = uri
        self.body = body
        self.headers = headers
        parsed = urllib.parse.urlparse(uri)
        params = urllib.parse.parse_qs(parsed.query)

        self.limit = int(params['limit'][0])

        # offset is always present with list-based paging but cannot be
        # present on the initial request with cursor-based paging
        self.offset = int(params.get('offset', [0])[0])

class MockAlternatePagingHTTPConnection(MockPagingHTTPConnection):
    def read(self):
        metadata = {}
        metadata['total_objects'] = len(self.objects)
        if self.offset + self.limit < len(self.objects):
            metadata['next_offset'] = self.offset + self.limit
        if self.offset > 0:
            metadata['prev_offset'] = max(self.offset-self.limit, 0)

        return json.dumps(
                {"stat":"OK",
                 "response": {
                    "data" : self.objects[self.offset: self.offset+self.limit],
                    "metadata": metadata
                  },
                },
                cls=MockObjectJsonEncoder)


class MockMultipleRequestHTTPConnection(MockHTTPConnection):
    def __init__(self, statuses):
        super(MockMultipleRequestHTTPConnection, self).__init__()
        self.statuses = statuses
        self.status_iterator = iter(statuses)
        self.requests = 0
        self.status = None

    def read(self):
        response = {'foo': 'bar'}
        return json.dumps({"stat":"OK", "response":response},
                              cls=MockObjectJsonEncoder)

    def request(self, method, uri, body, headers):
        self.requests += 1
        self.status = next(self.status_iterator)
        super(MockMultipleRequestHTTPConnection, self).request(
            method, uri, body, headers)



from typing import Dict, Sequence, Tuple
from datetime import datetime, timedelta, timezone


def get_params_from_kwargs(valid_params: Sequence[str], **kwargs) -> Dict:
    params = {}
    for k in kwargs:
        if kwargs[k] is not None and k in valid_params:
            params[k] = kwargs[k]
    return params


def get_log_uri(log_type: str, version: int = 1) -> str:
    return f"/admin/v{version}/logs/{log_type}"


def get_default_request_times() -> Tuple[int, int]:
    today = datetime.now(tz=timezone.utc)
    mintime = int((today - timedelta(days=180)).timestamp() * 1000)
    maxtime = int(today.timestamp() * 1000) - 120
    return mintime, maxtime



"""
Example of Duo Auth API with asynchronous user authentication action

This example uses the threading and queue libraries to illustrate how multiple users could potentially have
authentication requests in flight at the same time while the application polls for responses on each authentication
event without blocking main program execution.
"""
import getpass
import logging
import queue
import signal
import os
import sys
import time
import threading
from pathlib import Path
import traceback
from logging.handlers import RotatingFileHandler
from datetime import datetime

from duo_client import Auth

FIVE_MINUTES = 5 * 60
WORKER_THREADS = 3
SHUTDOWN_TIMEOUT = 10
WORKER_SLEEP_INTERVAL = 0.5


def _write_auth_entry(auth_entry: dict) -> None:
    """Write authentication result entry to separate log file"""
    filename = Path(__file__).with_name("user_authentication_result.log")
    human_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    with open(filename, 'a', encoding='utf-8') as auth_fn:
        auth_fn.write(f"{human_time} - {auth_entry}\n")


class DuoAuthAPI:
    """
    Class to hold global variables and methods used by the Duo Auth
    """

    def __init__(self):
        """Setup Duo Auth API object"""
        self.RUNNING = True
        self.DEBUG = True
        self.lock = threading.Lock()
        signal.signal(signal.SIGINT, self.close)
        self.logger = self._init_logger()
        self.logger.info(f"========== Starting {Path(__file__).name} ==========")
        self.stderr_tmp = sys.stderr
        sys.stderr = open(os.devnull, 'w')

        credentials = self.prompt_for_credentials()

        self._auth_client = Auth(
                ikey=credentials['IKEY'],
                skey=credentials['SKEY'],
                host=credentials['APIHOST']
        )
        if not self.ping_duo():
            self.exit_with_error("Duo Ping failed.")
        if not self.verify_duo():
            self.exit_with_error("Unable to verify Duo Auth API credentials.")

        self.authentications = {}
        """
        self.authentications[txid] = {
                  "timestamp": int,
                  "username": str,
                  "success": bool,
                  "status": str,
                  "message": str
        """

        self.user_queue = queue.Queue()
        self.auth_queue = queue.Queue()
        self.result_queue = queue.Queue()

        self.initialize_threads()

    @staticmethod
    def _init_logger():
        logger = logging.getLogger(__name__)
        f = Path(__file__)
        log_handler = RotatingFileHandler(
                filename=f.with_name(f.stem + ".log"),
                maxBytes=25000000,
                backupCount=5
        )
        LOGGING_FORMAT = "{asctime} [{levelname}]\t{module} : {funcName}({lineno}) - {message}"
        log_handler.setFormatter(logging.Formatter(LOGGING_FORMAT, style='{'))
        logger.addHandler(log_handler)
        logger.setLevel(logging.DEBUG)
        logger.info(f"Logger created with file {f.with_name(f.stem + '.log')} at log level " +
                    f"{logging.getLevelName(logger.getEffectiveLevel())}")
        return logger

    @staticmethod
    def _get_user_input(prompt, secure=False):
        """Read information from STDIN, using getpass when sensitive information should not be echoed to tty"""
        if secure is True:
            return getpass.getpass(prompt)
        else:
            return input(prompt)

    def close(self, signal_number, frame):
        """
        Handle CRTL-C interrupt signal and exit program
        """
        if self.DEBUG is True:
            self.logger.debug(f"Signal number {signal_number} received.")
            self.logger.debug(f"Frame traceback: {traceback.print_stack(frame)}")
        self.logger.info(f"SIGINIT received. Waiting for threads to complete...")
        print(f"\n\nSIGINIT received. Waiting for threads to complete...\n")
        self.logger.info("Setting instance RUNNING property to False...")
        self.RUNNING = False
        for thread in threading.enumerate():
            if thread != threading.main_thread():
                self.logger.info(f"Waiting for {thread.name} thread to complete...")
                print(f"{thread.name} shutting down...")
                thread.join(timeout=SHUTDOWN_TIMEOUT)
                if thread.is_alive() is True:
                    self.logger.info(f"{thread.name} did not shut down gracefully.")
                    print(f"    {thread.name} did not shut down gracefully.")
        self.logger.info(f"All threads complete. Shutting down.")
        sys.stderr = self.stderr_tmp
        print(f"All threads complete. Shutting down.")
        sys.exit()

    def exit_with_error(self, reason: str) -> None:
        """Log error message and exit program"""
        self.logger.error(f"Exiting with error: {reason}")
        sys.exit()

    def ping_duo(self) -> bool:
        """Verify that the Duo service is available"""
        duo_ping = self._auth_client.ping()
        if 'time' in duo_ping:
            self.logger.info("Duo service check completed successfully.")
            return True
        else:
            self.logger.error(f"Error: {duo_ping}")
            return False

    def verify_duo(self) -> bool:
        """Verify that IKEY and SKEY information provided are valid"""
        duo_check = self._auth_client.check()
        if 'time' in duo_check:
            self.logger.info("IKEY and SKEY provided have been verified.")
            return True
        else:
            self.logger.error(f"Error: {duo_check}")
            return False

    def _cleanup_authentications_dictionary(self):
        """Background task to remove old data from authentications dictionary"""
        t_name = threading.current_thread().name
        self.logger.info(f"#### Starting thread {t_name} ####")
        while self.RUNNING is True:
            threshold_time = int(time.time()) - FIVE_MINUTES
            self.logger.info(f"[{t_name}] Scanning for authentication data for older than {threshold_time}")
            self.lock.acquire(blocking=True)
            try:
                for txid in list(self.authentications.keys()):
                    if self.authentications[txid]['timestamp'] < threshold_time:
                        self.logger.warning(f"[{t_name}] *** Removing {txid} from authentications dictionary ***")
                        del self.authentications[txid]
            finally:
                self.lock.release()
            time.sleep(30)
        self.logger.info(f"[{t_name}] RUNNING property set to False. Cleaning up...")

    def prompt_for_credentials(self) -> dict:
        """Collect required API credentials from command line prompts and return them in a dictionary format"""
        ikey = self._get_user_input('Duo Auth API integration key ("DI..."): ')
        skey = self._get_user_input('Duo Auth API integration secret key: ', secure=True)
        host = self._get_user_input('Duo Auth API hostname ("api-....duosecurity.com"): ')
        return {"IKEY": ikey, "SKEY": skey, "APIHOST": host}

    def prompt_for_username(self) -> None:
        """Collect username from TTY and place on preauth_queue."""
        self.logger.debug(f"Prompting for username...") if self.DEBUG is True else ...
        username = self._get_user_input("Duo username to authenticate: ")
        self.logger.debug(f"    Username: {username} received") if self.DEBUG is True else ...
        self.user_queue.put_nowait(username)
        self.logger.info(f"    {username} placed in user_queue.")

    def preauth_user_from_queue(self) -> None:
        """Preauth user from pre-auth queue"""
        t_name = threading.current_thread().name
        self.logger.info(f"#### Starting thread {t_name} ####")
        duo_user = None
        got_item = False
        while self.RUNNING is True and got_item is False:
            try:
                duo_user = self.user_queue.get(block=False)
                got_item = True
            except queue.Empty:
                time.sleep(WORKER_SLEEP_INTERVAL)
            if got_item is False:
                continue
            self.logger.info(f"[{t_name}] Executing pre-authentication for {duo_user}...")
            pre_auth = self._auth_client.preauth(duo_user)
            self.logger.info(f"[{t_name}] Pre-authentication result for {duo_user} is {pre_auth}")
            if pre_auth['result'] == 'auth':
                self.auth_queue.put_nowait((duo_user, pre_auth))
                self.user_queue.task_done()
            else:
                self.logger.error(f"[{t_name}] Pre-auth for {duo_user} failed. Reason: {pre_auth}")
        self.logger.info(f"[{t_name}] RUNNING property set to False. Cleaning up...")

    def auth_user_from_queue(self) -> None:
        """Authenticate user from pre-auth queue"""
        t_name = threading.current_thread().name
        self.logger.info(f"#### Starting thread {t_name} ####")
        duo_user = None
        got_item = False
        while self.RUNNING is True and got_item is False:
            try:
                (duo_user, pre_auth_result) = self.auth_queue.get(block=False)
                got_item = True
            except queue.Empty:
                time.sleep(WORKER_SLEEP_INTERVAL)
            if got_item is False:
                continue
            try:
                self.logger.info(f"[{t_name}] Executing asynchronous authentication action for {duo_user}...")
                auth = self._auth_client.auth(factor="push", username=duo_user, device="auto", async_txn=True)
                if 'txid' in auth:
                    self.logger.info(f"[{t_name}] Placing {auth['txid']} in result_queue for user {duo_user}")
                    self.result_queue.put_nowait((duo_user, auth['txid']))
                    self.auth_queue.task_done()
            except Exception as e_str:
                self.logger.exception(f"[{t_name}] Exception caught: {e_str}")
        self.logger.info(f"[{t_name}] RUNNING property set to False. Cleaning up...")

    def get_user_auth_result(self) -> None:
        """Gets user authentication result from result_queue"""
        t_name = threading.current_thread().name
        self.logger.info(f"#### Starting thread {t_name} ####")
        duo_user = None
        txid = None
        got_item = False
        while self.RUNNING is True and got_item is False:
            try:
                (duo_user, txid) = self.result_queue.get(block=False)
                got_item = True
            except queue.Empty:
                time.sleep(WORKER_SLEEP_INTERVAL)
            if got_item is False:
                continue
            self.logger.info(f"[{t_name}] Getting authentication result for TXID {txid}, username {duo_user}...")
            waiting = True
            while waiting is True:
                self.logger.info(f"[{t_name}] Waiting for {duo_user} to respond [{txid}]...")
                auth_status = self._auth_client.auth_status(txid)
                if auth_status['waiting'] is not True:
                    waiting = False
                    self.logger.info(f"[{t_name}] Authentication result for {duo_user} [{txid}] is {auth_status}")
                    # Record authentication result for potential use elsewhere in the program
                    self.lock.acquire()
                    try:
                        self.authentications[txid] = {
                                "timestamp": int(time.time()),
                                "message":   auth_status['status_msg'],
                                "status":    auth_status['status'],
                                "success":   auth_status['success'],
                                "username":  duo_user
                        }
                    finally:
                        self.lock.release()
                    _write_auth_entry(self.authentications[txid])
                else:
                    self.logger.info(f"[{t_name}] Still waiting for {duo_user} to respond [{auth_status}]")
            self.result_queue.task_done()
        self.logger.info(f"[{t_name}] RUNNING property set to False. Cleaning up...")

    def initialize_threads(self):
        """Start background worker threads to monitor queues and process items"""
        threading.Thread(target=self._cleanup_authentications_dictionary,
                         name="Auth-dict-cleanup",
                         daemon=True).start()
        for i in range(WORKER_THREADS):
            threading.Thread(target=self.preauth_user_from_queue,
                             name=f"Pre-auth-worker-{i}",
                             daemon=True).start()
            threading.Thread(target=self.auth_user_from_queue,
                             name=f"Auth-worker-{i}",
                             daemon=True).start()
            threading.Thread(target=self.get_user_auth_result,
                             name=f"Result-worker-{i}",
                             daemon=True).start()

    def run(self):
        """Run the program setup and loop"""
        while self.RUNNING is True:
            self.prompt_for_username()
            time.sleep(1)
        self.logger.info(f"[run()] RUNNING property set to False. Cleaning up...")
        sys.stderr = self.stderr_tmp


if __name__ == '__main__':
    duo_auth_api = DuoAuthAPI()
    duo_auth_api.run()




"""
Example of Duo Auth API uaer authentication with synchronous request/response using an assigned token
as the MFA factor
"""

import duo_client
import sys
import getpass

from pprint import pprint


argv_iter = iter(sys.argv[1:])


def _get_next_arg(prompt, secure=False):
    """Read information from STDIN, using getpass when sensitive information should not be echoed to tty"""
    try:
        return next(argv_iter)
    except StopIteration:
        if secure is True:
            return getpass.getpass(prompt)
        else:
            return input(prompt)


def prompt_for_credentials() -> dict:
    """Collect required API credentials from command line prompts

    :return: dictionary containing Duo Auth API ikey, skey and hostname strings
    """

    ikey = _get_next_arg('Duo Auth API integration key ("DI..."): ')
    skey = _get_next_arg('Duo Auth API integration secret key: ', secure=True)
    host = _get_next_arg('Duo Auth API hostname ("api-....duosecurity.com"): ')
    username = _get_next_arg('Duo Username: ')

    return {"USERNAME": username, "IKEY": ikey, "SKEY": skey, "APIHOST": host}


def main():
    """Main program entry point"""

    inputs = prompt_for_credentials()

    auth_client = duo_client.Auth(
            ikey=inputs['IKEY'],
            skey=inputs['SKEY'],
            host=inputs['APIHOST']
    )

    # Verify that the Duo service is available
    duo_ping = auth_client.ping()
    if 'time' in duo_ping:
        print("\nDuo service check completed successfully.")
    else:
        print(f"Error: {duo_ping}")

    # Verify that IKEY and SKEY information provided are valid
    duo_check= auth_client.check()
    if 'time' in duo_check:
        print("IKEY and SKEY provided have been verified.")
    else:
        print(f"Error: {duo_check}")

    # Execute pre-authentication for given user
    print(f"\nExecuting pre-authentication for {inputs['USERNAME']}...")
    pre_auth = auth_client.preauth(username=inputs['USERNAME'])

    print("\n" + "=" * 30)
    pprint(f"Pre-Auth result: {pre_auth}")
    print("=" * 30 + "\n")

    for device in pre_auth['devices']:
        pprint(device)
        print()

    if pre_auth['result'] == "auth":
        try:
            print(f"Executing authentication action for {inputs['USERNAME']}...")
            # Prompt for the hardware token passcode
            passcode = _get_next_arg('Duo token passcode: ')
            auth = auth_client.auth(factor="passcode", username=inputs['USERNAME'], passcode=passcode)
            print(f"\n{auth['status_msg']}")
        except Exception as e_str:
            print(e_str)
    else:
        print(pre_auth)


if __name__ == '__main__':
    main()



from typing import Callable
from duo_client.util import (
    get_params_from_kwargs,
    get_log_uri,
    get_default_request_times,
)

VALID_TELEPHONY_V2_REQUEST_PARAMS = [
    "filters",
    "mintime",
    "maxtime",
    "limit",
    "sort",
    "next_offset",
    "account_id",
]

LOG_TYPE = "telephony"


class Telephony:
    @staticmethod
    def get_telephony_logs_v1(json_api_call: Callable, host: str, mintime=0):
        # Sanity check mintime as unix timestamp, then transform to string
        mintime = f"{int(mintime)}"
        params = {
            "mintime": mintime,
        }
        response = json_api_call(
            "GET",
            get_log_uri(LOG_TYPE, 1),
            params,
        )
        for row in response:
            row["eventtype"] = LOG_TYPE
            row["host"] = host
        return response

    @staticmethod
    def get_telephony_logs_v2(json_api_call: Callable, host: str, **kwargs):
        params = {}
        default_mintime, default_maxtime = get_default_request_times()

        params = get_params_from_kwargs(VALID_TELEPHONY_V2_REQUEST_PARAMS, **kwargs)

        if "mintime" not in params:
            # If mintime is not provided, the script defaults it to 180 days in past
            params["mintime"] = default_mintime
        params["mintime"] = f"{int(params['mintime'])}"
        if "maxtime" not in params:
            # if maxtime is not provided, the script defaults it to now
            params["maxtime"] = default_maxtime
        params["maxtime"] = f"{int(params['maxtime'])}"
        if "limit" in params:
            params["limit"] = f"{int(params['limit'])}"

        response = json_api_call(
            "GET",
            get_log_uri(LOG_TYPE, 2),
            params,
        )
        for row in response["items"]:
            row["eventtype"] = LOG_TYPE
            row["host"] = host
        return response



from .base import TestAdmin
from .. import util


class TestRegisteredDevices(TestAdmin):
    def test_get_registered_devices_generator(self):
        """ Test to get desktop tokens generator.
        """
        generator = self.client_list.get_registered_devices_generator()
        response = next(generator)
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/registered_devices')
        self.assertEqual(util.params_to_dict(args),
                {'account_id': [self.client_list.account_id], 'limit': ['100'], 'offset': ['0'], })

    def test_get_registered_devices(self):
        """ Test to get desktop tokens without params.
        """
        response = self.client_list.get_registered_devices()[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/registered_devices')
        self.assertEqual(util.params_to_dict(args),
                {'account_id': [self.client_list.account_id], 'limit': ['100'], 'offset': ['0'], })

    def test_get_registered_devices_limit(self):
        """ Test to get desktop tokens with limit.
        """
        response = self.client_list.get_registered_devices(limit='20')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/registered_devices')
        self.assertEqual(util.params_to_dict(args),
                {'account_id': [self.client_list.account_id], 'limit': ['20'], 'offset': ['0'], })

    def test_get_registered_devices_offset(self):
        """ Test to get desktop tokens with offset.
        """
        response = self.client_list.get_registered_devices(offset='20')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/registered_devices')
        self.assertEqual(util.params_to_dict(args),
                {'account_id': [self.client_list.account_id], 'limit': ['100'], 'offset': ['0'], })

    def test_get_registered_devices_limit_offset(self):
        """ Test to get desktop tokens with limit and offset.
        """
        response = self.client_list.get_registered_devices(limit='20', offset='2')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/registered_devices')
        self.assertEqual(util.params_to_dict(args),
                {'account_id': [self.client_list.account_id], 'limit': ['20'], 'offset': ['2'], })

    def test_delete_registered_device(self):
        """ Test to delete registered device by registered device id.
        """
        response = self.client.delete_registered_device('CRSFWW1YWVNUXMBJ1J29')
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'DELETE')
        self.assertEqual(uri, '/admin/v1/registered_devices/CRSFWW1YWVNUXMBJ1J29')
        self.assertEqual(util.params_to_dict(args), {'account_id': [self.client.account_id]})



from setuptools import setup

import os.path

import duo_client

requirements_filename = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "requirements.txt"
)

with open(requirements_filename) as fd:
    install_requires = [i.strip() for i in fd.readlines()]

requirements_dev_filename = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "requirements-dev.txt"
)

with open(requirements_dev_filename) as fd:
    tests_require = [i.strip() for i in fd.readlines()]

long_description_filename = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "README.md"
)

with open(long_description_filename) as fd:
    long_description = fd.read()

setup(
    name="duo_client",
    version=duo_client.__version__,
    description="Reference client for Duo Security APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Duo Security, Inc.",
    author_email="support@duosecurity.com",
    url="https://github.com/duosecurity/duo_client_python",
    packages=["duo_client", "duo_client.logs"],
    package_data={"duo_client": ["ca_certs.pem"]},
    license="BSD",
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
    ],
    install_requires=install_requires,
    tests_require=tests_require,
)



#!/usr/bin/env python
import sys
from datetime import datetime, timezone

import duo_client

argv_iter = iter(sys.argv[1:])


def get_next_input(prompt):
    """Collect user input from terminal and return it."""
    try:
        return next(argv_iter)
    except StopIteration:
        return input(prompt)


def human_time(time: int) -> str:
    """Translate unix time into human readable string"""
    if time is None:
        date_str = 'Never'
    else:
        date_str = datetime.fromtimestamp(time, timezone.utc).strftime("%Y-%m-%m %H:%M:%S")
    return date_str


# Configuration and information about objects to create.
admin_api = duo_client.Admin(
        ikey=get_next_input('Admin API integration key ("DI..."): '),
        skey=get_next_input('integration secret key: '),
        host=get_next_input('API hostname ("api-....duosecurity.com"): '), )

# Retrieve user info from API:
users = admin_api.get_users()

print(f'{"Username":^30} {"Last Login":^20} {"User Enrolled"}')
print(f'{"=" * 30} {"=" * 20} {"=" * 15}')
for user in users:
    line_out = f"{user['username']:30} "
    line_out += f"{human_time(user['last_login']):20} "
    line_out += f" {user['is_enrolled']} "
    print(line_out)



import json
import warnings
from .. import util
import duo_client.admin
from .base import TestAdmin


class TestGroups(TestAdmin):
    def test_get_groups_generator(self):
        """ Test to get groups generator.
        """
        generator = self.client_list.get_groups_generator()
        response = next(generator)
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/groups')
        self.assertEqual(
            util.params_to_dict(args),
            {
                'account_id': [self.client_list.account_id],
                'limit': ['100'],
                'offset': ['0'],
            })

    def test_get_groups(self):
        """ Test to get groups without params.
        """
        response = self.client_list.get_groups()[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/groups')
        self.assertEqual(
            util.params_to_dict(args),
            {
                'account_id': [self.client_list.account_id],
                'limit': ['100'],
                'offset': ['0'],
            })

    def test_get_groups_limit(self):
        """ Test to get groups with limit.
        """
        response = self.client_list.get_groups(limit='20')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/groups')
        self.assertEqual(
            util.params_to_dict(args),
            {
                'account_id': [self.client_list.account_id],
                'limit': ['20'],
                'offset': ['0'],
            })

    def test_get_groups_offset(self):
        """ Test to get groups with offset.
        """
        response = self.client_list.get_groups(offset='2')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/groups')
        self.assertEqual(
            util.params_to_dict(args),
            {
                'account_id': [self.client_list.account_id],
                'limit': ['100'],
                'offset': ['0'],
            })

    def test_get_groups_limit_offset(self):
        """ Test to get groups with limit and offset.
        """
        response = self.client_list.get_groups(limit='20', offset='2')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v1/groups')
        self.assertEqual(
            util.params_to_dict(args),
            {
                'account_id': [self.client_list.account_id],
                'limit': ['20'],
                'offset': ['2'],
            })

    def test_get_group_v1(self):
        """ Test for v1 API of getting specific group details
        """
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            response = self.client.get_group('ABC123', api_version=1)
            uri, args = response['uri'].split('?')

            # Assert deprecation warning generated
            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))
            self.assertIn('Please migrate to the v2 API', str(w[0].message))

            self.assertEqual(response['method'], 'GET')
            self.assertEqual(uri, '/admin/v1/groups/ABC123')
            self.assertEqual(util.params_to_dict(args),
                             {'account_id': [self.client.account_id]})

    def test_get_group_v2(self):
        """ Test for v2 API of getting specific group details
        """
        response = self.client.get_group('ABC123', api_version=2)
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v2/groups/ABC123')
        self.assertEqual(util.params_to_dict(args),
                         {'account_id': [self.client.account_id]})

    def test_get_group_users(self):
        """ Test for getting list of users associated with a group
        """
        response = self.client_list.get_group_users('ABC123')[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v2/groups/ABC123/users')
        self.assertEqual(
            util.params_to_dict(args),
            {
                'account_id': [self.client.account_id],
                'limit': ['100'],
                'offset': ['0'],
            })

    def test_get_group_users_with_offset(self):
        """Test to get users by group id with pagination params
        """
        response = self.client_list.get_group_users('ABC123', offset=30)[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v2/groups/ABC123/users')
        self.assertEqual(util.params_to_dict(args),
                         {
                            'account_id':[self.client.account_id],
                            'limit': ['100'],
                            'offset': ['0'],
                         })

    def test_get_group_users_with_limit(self):
        """Test to get users by group id with pagination params
        """
        response = self.client_list.get_group_users('ABC123', limit=30)[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v2/groups/ABC123/users')
        self.assertEqual(util.params_to_dict(args),
                         {
                            'account_id':[self.client.account_id],
                            'limit': ['30'],
                            'offset': ['0'],
                         })

    def test_get_group_users_with_limit_and_offset(self):
        """Test to get users by group id with pagination params
        """
        response = self.client_list.get_group_users(
            'ABC123', limit=30, offset=60)[0]
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v2/groups/ABC123/users')
        self.assertEqual(util.params_to_dict(args),
                         {
                            'account_id':[self.client.account_id],
                            'limit': ['30'],
                            'offset': ['60'],
                         })

    def test_get_group_users_iterator(self):
        """Test to get user iterator by group id
        """
        iterator = self.client_list.get_group_users_iterator(
            'ABC123')
        response = next(iterator)
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'GET')
        self.assertEqual(uri, '/admin/v2/groups/ABC123/users')
        self.assertEqual(util.params_to_dict(args),
                         {
                            'account_id':[self.client.account_id],
                            'limit': ['100'],
                            'offset': ['0'],
                         })

    def test_delete_group(self):
        """ Test for deleting a group
        """
        response = self.client.delete_group('ABC123')
        uri, args = response['uri'].split('?')

        self.assertEqual(response['method'], 'DELETE')
        self.assertEqual(uri, '/admin/v1/groups/ABC123')
        self.assertEqual(util.params_to_dict(args),
                         {'account_id': [self.client.account_id]})

    def test_modify_group(self):
        """ Test for modifying a group
        """
        response = self.client.modify_group('ABC123')
        self.assertEqual(response['method'], 'POST')
        self.assertEqual(response['uri'], '/admin/v1/groups/ABC123')
        self.assertEqual(json.loads(response['body']),
                         {'account_id': self.client.account_id})
